Package: maxLik
Version: 1.3-4
Date: 2015-11-08
Title: Maximum Likelihood Estimation and Related Tools
Author: Ott Toomet <otoomet@gmail.com>,
   Arne Henningsen <arne.henningsen@gmail.com>,
   with contributions from Spencer Graves and Yves Croissant
Maintainer: Ott Toomet <otoomet@gmail.com>
Depends: R (>= 2.4.0), miscTools (>= 0.6-8), methods
Imports: sandwich
Description: Functions for Maximum Likelihood (ML) estimation and non-linear
   optimization, and related tools.  It includes a unified way to call
   different optimizers, and classes and methods to handle the results from
   the ML viewpoint.  It also includes a number of convenience tools for testing
   and developing your own models.
License: GPL (>= 2)
ByteCompile: yes
URL: http://www.maxLik.org
NeedsCompilation: no
Packaged: 2015-11-09 03:33:54 UTC; siim
Repository: CRAN
Date/Publication: 2015-11-09 14:41:52
Built: R 3.4.1; ; 2017-10-22 14:25:05 UTC; windows
