export( "activePar" )
export( "compareDerivatives" )
export( "condiNumber" )
export( "fnSubset" )
export( "hessian" )
export( "maxBFGS" )
export( "maxBFGSR" )
export( "maxBHHH" )
export( "maxCG" )
export( "maximType" )
export( "maxLik" )
export( "maxNM" )
export( "maxNR" )
export( "maxSANN" )
export( "nIter" )
export( "numericGradient" )
export( "numericHessian" )
export( "numericNHessian" )
export( "returnCode" )
export( "returnMessage" )
export("sumt")

importFrom("methods", "new", "show", "slot", "slot<-", "slotNames",
           "validObject")
importFrom( "miscTools", "nObs" )
importFrom( "miscTools", "nParam" )
importFrom( "miscTools", "sumKeepAttr" )
importFrom( "sandwich", "bread" )
importFrom( "miscTools", "stdEr" )
importFrom( "sandwich", "estfun" )
importFrom("stats", "coef", "logLik", "optim", "pnorm", "printCoefmat",
           "vcov")
importFrom("utils", "head", "str", "tail")

exportClasses("MaxControl")
exportMethods("maxControl")
exportMethods("show")

S3method( "activePar", "default" )
S3method( "AIC", "maxLik" )
S3method( "bread", "maxLik" )
S3method( "coef", "maxim" )
S3method( "coef", "maxLik" )
S3method( "coef", "summary.maxLik" )
S3method( "condiNumber", "default" )
S3method( "condiNumber", "maxLik" )
S3method( "estfun", "maxLik" )
S3method( "hessian", "default" )
S3method( "logLik", "maxLik" )
S3method( "logLik", "summary.maxLik" )
S3method( "maximType", "default" )
S3method( "maximType", "maxim" )
S3method( "nIter", "default" )
S3method( "nObs", "maxLik" )
S3method( "nParam", "maxim" )
S3method( "print", "maxLik" )
S3method( "print", "summary.maxim" )
S3method( "print", "summary.maxLik" )
S3method( "returnCode", "default" )
S3method( "returnCode", "maxim" )
S3method( "returnCode", "maxLik" )
S3method( "returnMessage", "default" )
S3method( "returnMessage", "maxim" )
S3method( "returnMessage", "maxLik" )
S3method( "stdEr", "maxLik" )
S3method( "summary", "maxim" )
S3method( "summary", "maxLik" )
S3method( "vcov", "maxLik" )
